
#import "ZKHudView.h"
#import <UIKit/UIKit.h>
#import <MBProgressHUD.h>

#import <React/RCTConvert.h>

#define MBProgressHUDModeJsNameNativeValueMap @{@"MBProgressHUDModeIndeterminate":@(MBProgressHUDModeIndeterminate),\
@"MBProgressHUDModeDeterminate":@(MBProgressHUDModeDeterminate),\
@"MBProgressHUDModeDeterminateHorizontalBar":@(MBProgressHUDModeDeterminateHorizontalBar),\
@"MBProgressHUDModeAnnularDeterminate":@(MBProgressHUDModeAnnularDeterminate),\
@"MBProgressHUDModeCustomView":@(MBProgressHUDModeCustomView),\
@"MBProgressHUDModeText":@(MBProgressHUDModeText),}

@interface RCTConvert (HudType)
@end

@implementation RCTConvert (HudType)
RCT_ENUM_CONVERTER(MBProgressHUDMode,
                   (MBProgressHUDModeJsNameNativeValueMap),
                   MBProgressHUDModeText,
                   integerValue)
@end

@interface ZKHudView () <MBProgressHUDDelegate>

@property (strong, nonatomic) MBProgressHUD *hud;

@property (strong, nonatomic) NSMutableArray *hideCallbacks;

@end

@implementation ZKHudView

- (instancetype)init {
  self = [super init];
  if (self) {
    _hideCallbacks = [NSMutableArray array];
  }
  return self;
}

RCT_EXPORT_MODULE();

RCT_EXPORT_METHOD(showHudMode:(MBProgressHUDMode)mode labelText:(NSString*)labelText hideAfter:(int)dur callback:(RCTResponseSenderBlock)callback) {
  [self showHudModeImp:mode labelText:labelText hideAfter:dur callback:callback];
}

- (void)showHudModeImp:(MBProgressHUDMode)mode
             labelText:(NSString*)labelText
             hideAfter:(int)dur
              callback:(RCTResponseSenderBlock)callback {
  dispatch_async(dispatch_get_main_queue(), ^{
    UIWindow *window = [UIApplication sharedApplication].keyWindow;
    if (!self.hud) {
      MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:window animated:YES];
      hud.delegate = self;
      hud.bezelView.backgroundColor = [UIColor colorWithRed:0 green:0 blue:0 alpha:0.7];
      hud.contentColor = [UIColor whiteColor];
      self.hud = hud;
    }
    self.hud.label.text = labelText ?: @"";
    self.hud.mode = mode;
    [window addSubview:self.hud];
    [self.hud showAnimated:YES];
    
    if (dur > 0) {
      [self.hud hideAnimated:YES afterDelay:dur];
      if (callback) {
        [self.hideCallbacks addObject:callback];
      }
    } else {
      if (callback) {
        callback(@[[NSNull null]]);
      }
    }
  });
}

RCT_EXPORT_METHOD(hideHud:(BOOL)animated callback:(RCTResponseSenderBlock)callback) {
  dispatch_async(dispatch_get_main_queue(), ^{
    if (callback) {
      [self.hideCallbacks addObject:callback];
    }
    [self.hud hideAnimated:animated];
  });
}

RCT_EXPORT_METHOD(updateText:(NSString*)title hideAfter:(int)dur callback:(RCTResponseSenderBlock)callback ) {
  [self showHudMode:MBProgressHUDModeText labelText:title hideAfter:dur callback:callback];
}

- (void)hudWasHidden:(MBProgressHUD *)hud {
  if (hud == self.hud) {
    NSArray *callbacks = self.hideCallbacks;
    self.hideCallbacks = [NSMutableArray array];
    [callbacks enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
      ((RCTResponseSenderBlock)obj)(@[[NSNull null]]);
    }];
  }
}

- (NSDictionary*)constantsToExport {
  NSMutableDictionary *ret = [NSMutableDictionary dictionary];
  [ret addEntriesFromDictionary:MBProgressHUDModeJsNameNativeValueMap];
  return ret;
}

@end
  
